/*
  deleting.c

  includes procedures for deleting FlowGraph's and ParseTree's 

*/

#include "deleting.h"
#include "parsercontrol.h"
#include "symboltable.h"

extern SymbolTable vartable;

/*
  void deleteFlowNodeList(FlowNodeList l)
  deletes the given FlowNodeList l and frees the used memory
*/
void deleteFlowNodeList(FlowNodeList l)
{
  FlowNodeList buffer = l;
  while (l != NULL)
    {
      buffer = l->next;
      free((FlowNodeList)l);
      l = buffer;
    }
}

void deleteFlowNode(FlowNode f, FrameWork fw, int free_elems)
{
    if (f == NULL) return;
    deleteFlowNodeList(f->entry);
    deleteFlowNodeList(f->exit);
    
    if (free_elems)
	fw->deref(f->analysis);
    free(f);
   
}

/*
  void deleteFlowGraph(FlowGraph g, FrameWork fw, int free_elems)
  deletes the given FlowGraph g and frees the used memory
*/
void deleteFlowGraph(FlowGraph g, FrameWork fw, int free_elems)
{
    if (g == NULL) return;
    FlowNodeList walker, buffer;
    walker = g->all;
    while (walker != NULL)
    {
	buffer = walker->next;
     	deleteFlowNode(walker->node, fw, free_elems);
	walker = buffer;
    }
    deleteFlowNodeList(g->exit);
    deleteFlowNodeList(g->all);
    free(g);
    
}


/*
  void deleteFlowGraphElements(FlowGraph g)
  deletes the the FlowGraph (framework) elements
*/
void deleteFlowGraphElements(FlowGraph g, FrameWork fw)
{
  if (g == NULL) return;
  FlowNodeList walker = g->all;
  while (walker != NULL)
    {
	fw->deref(walker->node->analysis);
	walker->node->analysis = NULL;
	walker = walker->next;
	}
}

/*
  void deleteParseTree(ParseTree tree)
  frees the memory used by the given ParseTree
*/
void deleteParseTree(ParseTree tree)
{
     ParseTreeCell w,h;
    if (tree == NULL) return;
    
    if ((tree->type == INTEGER) ||
	(tree->type == VARIABLE) ||
	(tree->type == TRUE) ||
	(tree->type == FALSE) ||
	(tree->type == SKIP))
    {
	free(tree);
    }
    else
    {
	
	w = tree->tag.subtrees->first;
	while (w != NULL)
	{
	    h = w->next;
	    deleteParseTree(w->value);
	    free(w);
	    w = h;
	}
	}
    
}

/*
  void deleteSymbolTable(SymbolTable t)
  frees the symbol table
*/
void deleteSymbolTable(SymbolTable t)
{
    if (t == NULL) return;
    SymbolCell w = t->entries, h;
    while (w != NULL)
    {
	h = w->next;
        free(w->name); /* deleting the string */
	free(w);
	w = h;
    }
    free(t);
}

/*
  void deleteLabelList(LabelList l)
  frees the LabelList l
*/
void deleteLabelList(LabelList l)
{
  if (l == NULL) return;
  LabelList h;
  while (l != NULL)
    {
      h = l->next;
      free(l->name); /* deleting the string */
      free(l);
      l = h;
    }
}
