/* flowrel_list.h
  
  Implements an List of FlowRelations, which is used for the worklist
  of the analysis algorithm
  
  11.6.2004
   
*/

#ifndef _FLOWREL_LIST_H
#define _FLOWREL_LIST_H

#include "flownode.h"
#include <stdlib.h>


/* (struct) FlowRelationElement
   Simple pair of FlowNode pointers
*/
typedef struct FlowRelationElement_
{
    FlowNode from, to;
} FlowRelationElement;

typedef struct FlowRelationList_
{
  FlowRelationElement value;
  struct FlowRelationList_ *next;
}*FlowRelationList;


/*
  FlowRelationList FlowRelationListCreate(FlowGraph f, char direction)
  creates an FlowRelationList which includes all transisitions of the given
  FlowGraph f (which given direction: 1 bottom up, else top down)
*/
FlowRelationList FlowRelationListCreate(FlowGraph f, char direction);

/*
  int FlowRelationListNull(FlowRelationList l)
  Returns:
  1: |l| = 0
  0: else
*/
int FlowRelationListNull(FlowRelationList l);

/*
  FlowRelationElement FlowRelationListFirst(FlowRelationList l)
  First (CAR) operator for the FlowRelationList
*/
FlowRelationElement FlowRelationListFirst(FlowRelationList l);

/*
  FlowRelationList FlowRelationListRest(FlowRelationList l)
  Rest (CDR) operator for the FlowRelationList
*/
FlowRelationList FlowRelationListRest(FlowRelationList l);

/*
  FlowRelationList FlowRelationListCons(FlowRelationElement el, FlowRelationList l)
  CONS operator for the FlowRelationList
*/
FlowRelationList FlowRelationListCons(FlowRelationElement el, FlowRelationList l);

#endif
