/* frame_managing.c
   Includes structures and procedures for handling the 
   different frameworks;
   each framework has to be "registered" in a FrameWorkList,
   which is actually a GenericList (int registerFrameWork) with
   FrameWorkEntry as data type. So it is possible to access the
   FrameWorks by name or by given id.
*/

#include "frame_managing.h"

/* (global) actual framework */
FrameWork actual_fw = NULL;


/* help functions */

int compareFrameWorkEntryName(Element a, Element b)
{
  if (strcmp(((FrameWorkEntry)(a))->name, ((FrameWorkEntry)(b))->name) > 0)
    return 0;
  else
      return 1;
}

int isequalFrameWorkEntryName(Element a, Element b)
{
    return strcmp(a, ((FrameWorkEntry)(b))->name) == 0;
}

int compareID(Element a, Element b)
{
  if ((int *)a > (int *)b)
    return 1;
  else
    return 0;
}

/* 
   void init_fwlist()
   initializes fwlist by registering all FrameWorks
*/
void init_fwlist()
{
  FrameWork buffer;
  fwlist = createFrameWorkList();     
  /* include FrameWork's here */
  
  registerFrameWork(fwlist, fw_simple_createFrameWork(0),  fw_simple_name);
  registerFrameWork(fwlist, fw_init_createFrameWork(0), fw_init_name);
  registerFrameWork(fwlist, fw_livevar_createFrameWork(1), fw_livevar_name);
  registerFrameWork(fwlist, fw_slivevar_createFrameWork(1), fw_slivevar_name);

}


/*
  GenericList createFrameWorkList()
  is mainly used for creating the global FrameWorkList fwlist
*/
GenericList createFrameWorkList()
{
  return createGenericList();
}


/*
  void registerFrameWork(GenericList fwList, FrameWork f, char *name)
  adds an entry to the given GenericList fwList with the given FramWork f and
  label name
*/
void registerFrameWork(GenericList fwList, FrameWork f, char *name)
{
  FrameWorkEntry newentry = (FrameWorkEntry)malloc(sizeof(struct FrameWorkEntry_));
  newentry->name = name;
  newentry->fw = f;
  GenericListAddElement(fwList, (void *)newentry,  &compareFrameWorkEntryName, INSERT_SORT);
  return;
}


/*
  FrameWork getFrameWorkbyName(GenericList fwList, char *name)
  searches an FrameWork with the given name in the given GenericList fwList
*/
FrameWork getFrameWorkbyName(GenericList fwList, char *name)
{
    FrameWorkEntry f;
    
    f = (FrameWorkEntry)(GenericListSearchElement(fwList, (void *)name, &isequalFrameWorkEntryName));
    if (f == NULL) return NULL;
    else return f->fw;
}

/*
  void setActualFrameWork(int nr)
  Sets the actual framework to the new analysis
  Important: there will be errors if there aren't
  (nr+1) frameworks in the fwlist !!!
*/
void setActualFrameWork(int nr)
{
    GenericCell walker = fwlist->first;
    if (nr == -1)
	actual_fw = NULL;
    else
    {
	while (walker != NULL)
	{
	    if (nr == 0)
	    {
		actual_fw = ((FrameWorkEntry)(walker->value))->fw;
		return;
	    }
	    else
	    {
		walker = walker->next;
		nr--;
	    }
	}
    }
}


FrameWork getActualFrameWork()
{
    return actual_fw;
}

/*
  TODO: LISTE DURCHLAUFEN MIT ORDNUNGSRELATION
*/

