/* frame_managing.h
   Includes structures and procedures for handling the 
   different frameworks;
   each framework has to be "registered" in a FrameWorkList,
   which is actually a GenericList (int registerFrameWork) with
   FrameWorkEntry as data type. So it is possible to access the
   FrameWorks by name or by given id.
*/

#ifndef _FRAME_MANAGING
#define _FRAME_MANAGING

#include <stdlib.h>
#include <string.h>
#include "framework.h"
#include "generic_list.h"



/* include new framework header files here: */
#include "fw/fw_simple.h"
#include "fw/fw_init.h"
#include "fw/fw_livevar.h"
#include "fw/fw_slivevar.h"

/* identifier strings */
extern char *fw_simple_name;
extern char *fw_init_name;
extern char *fw_livevar_name;
extern char *fw_slivevar_name;

typedef struct FrameWorkEntry_
{
  char *name;
  FrameWork fw;
} *FrameWorkEntry;


/* global FrameWorkList variable */
GenericList fwlist;

/* 
   void init_fwlist()
   initializes fwlist by registering all FrameWorks
*/
void init_fwlist();

/*
  GenericList createFrameWorkList()
  is mainly used for creating the global FrameWorkList fwlist
*/
GenericList createFrameWorkList();

/*
  void registerFrameWork(GenericList fwList, FrameWork f, char *name)
  adds an entry to the given GenericList fwList with the given FramWork f and
  label name
*/
void registerFrameWork(GenericList fwList, FrameWork f, char *name);

/*
  FrameWork getFrameWorkbyName(GenericList fwList, char *name)
  searches an FrameWork with the given name in the given GenericList fwList
*/
FrameWork getFrameWorkbyName(GenericList fwList, char *name); 

/*
  void setActualFrameWork(int nr)
  Sets the actual framework to the new analysis
  Important: there will be errors if there aren't
  (nr+1) frameworks in the fwlist !!!
*/
void setActualFrameWork(int nr);


FrameWork getActualFrameWork();


#endif
