/* framework.h
   18.05.2004
   data structures and functions
   for handling of frameworks and
   unions, which are used by the
   worklist algorithm
*/


#ifndef _FRAMEWORK_H
#define _FRAMEWORK_H

#include <stdio.h>
#include <stdlib.h>
#include "flownode.h"
#include "generic_list.h"


/*
  typedef's for the different 
  function pointers
*/
typedef Element (*Iota)();
typedef Element (*Supremum)(Element a, Element b);
typedef Element (*Bottom)();
typedef Element (*Transfer)(FlowNode g, Element a);
typedef int (*InitFunc)();
typedef void (*FinalizeFunc)();
typedef void (*DeRefElement)(Element a);
typedef Element (*RefElement)(Element a);
typedef int (*Eq)(Element a, Element b);
typedef char* (*PrintFunc)(Element a);

/*
  data structure for a general
  framework
*/
typedef struct FrameWork_
{
  /* direction "bit":
     1 for bottom up, 
     else top down
  */
  char direction;
  Iota iota;
  /* supremum relation pointer */
  Supremum supremum;
  /* bottom Element */
  Bottom bottom;
  /* the transfer function */ 
  Transfer transfer;
  /* equal relation */
  Eq eq;
  /* Dereffering an element */
  DeRefElement deref;  
  /* Referring an element */
  RefElement ref;
  /* Output function */
  PrintFunc printfunc;
  /* Initialization function */
  InitFunc initfunc;
  /* Finalizer function */
  FinalizeFunc finalizefunc;
  
} *FrameWork;


#endif
