/* fw_livevar.h

Living Variables
*/

#include "fw_livevar.h"
#include "../symboltable.h"
#include "../varset.h"

/* constants */
extern SymbolTable vartable;
extern void prompt(char *msg, char *end);

extern VarSet var_all;
char *fw_livevar_name = "Lebendige Variablen\0";

Element iotas;

/* iota Element  */
Element livevar_iota()
{
    return VarSetMerge(iotas, NULL);
}

Element livevar_supremum(Element a, Element b)
{
    return VarSetMerge((VarSet)a, (VarSet)b);
}

Element livevar_bottom()
{
    return (Element)NULL;
}

Element livevar_transfer(FlowNode g, Element a)
{
    VarSet result, buffer;
    int assigned = fwtools_assignedVariable(g->expr);
    if (assigned == -1)
    {
	if (g->type == SKIP)
	{
	    return (Element)VarSetMerge(g->analysis, NULL);
 	}
	if ((g->expr->type == IF) || (g->expr->type == WHILE))
	  {
          return (Element)VarSetMerge((VarSet)g->analysis, fwtools_allVariablesExpr(getSubTree(g->expr, 0)));
          }
    }
    else  /* assignment */
    {
	buffer = VarSetMerge((VarSet)g->analysis, NULL);
	buffer = VarSetremoveElement(buffer, assigned);
	result = VarSetMerge(buffer, fwtools_allVariablesExpr(getSubTree(g->expr, 1)));
	VarSetDestroy(buffer);
        return result;
    }
}

int livevar_eq(Element a, Element b)
{
    return VarSetEqual((VarSet)a, (VarSet)b);
}


void livevar_deref(Element a)
{
    VarSetDestroy((VarSet) a);
}


Element livevar_ref(Element a)
{
  return (Element)a;
}

char *livevar_print(Element a)
{
  return VarSetSPrint((VarSet) a);
}

int livevar_initfunc()
{
  char buffer[35];
  int id;
  char text[512] = "Ausgabevariablen auswaehlen aus der Variablenmenge ";
  int error = 0;

  iotas = (Element)createVarSet();
  
  prompt(strcat(text, VarSetSPrint(var_all)), "0");
  
  buffer[0] = '\n';
  while ((strcmp(buffer, "0")))
    {  
      scanf("%s", &buffer);
      if (!error)
      {
	  if (strcmp(buffer, "0"))   /* if  buffer != "0" */
	  {
	      toLower(buffer);
	      id = SymbolTableFindSymbolByName(vartable, buffer);
	      if (id != -1)
	      {
		  iotas = VarSetaddElement((VarSet)iotas, id);
	      }
	      else 
	      {
		  error = 1;
	      }
	  }
      }
    }

  if (error)
  {
      livevar_finalizefunc();
      return 0;	  
  }
  else
      return 1;
}

void livevar_finalizefunc()
{
  /* freeing the iotas element */
  VarSetDestroy(iotas);
}


FrameWork fw_livevar_createFrameWork(int direction)
{
    FrameWork result = (FrameWork)malloc(sizeof(struct FrameWork_));
    result->direction = direction;
    result->iota = (Iota)&livevar_iota;
    result->supremum = (Supremum)&livevar_supremum;
    result->bottom = (Bottom)&livevar_bottom;
    result->transfer = (Transfer)&livevar_transfer;
    result->eq = (Eq)&livevar_eq;
    result->ref = (RefElement)&livevar_ref;
    result->deref = (DeRefElement)&livevar_deref;
    result->printfunc = (PrintFunc)&livevar_print;
    result->initfunc = (InitFunc)&livevar_initfunc;
    result->finalizefunc = (FinalizeFunc)&livevar_finalizefunc;

    return result;
}
