/* fw_slivevar.h

Strong / Weak Living Variables
*/

#include "fw_slivevar.h"
#include "../symboltable.h"
#include "../varset.h"

/* constants */
extern SymbolTable vartable;

extern VarSet var_all;
char *fw_slivevar_name = "Stark Lebendige Variablen\0";

Element iotas;

/* iota Element  */
Element slivevar_iota()
{
    return VarSetMerge(iotas, NULL);
}


Element slivevar_supremum(Element a, Element b)
{
    return VarSetMerge((VarSet)a, (VarSet)b);
}

Element slivevar_bottom()
{
    return (Element)NULL;
}

Element slivevar_transfer(FlowNode g, Element a)
{
    VarSet result, buffer;
    int assigned = fwtools_assignedVariable(g->expr);
    if (assigned == -1)
    {
	if (g->type == SKIP)
	{
	    return (Element)VarSetMerge(g->analysis, NULL);
 	}
	if ((g->expr->type == IF) || (g->expr->type == WHILE))
	    return (Element)VarSetMerge((VarSet)g->analysis, fwtools_allVariablesExpr(getSubTree(g->expr, 0)));
    }
    else  /* assignment */
    {
      if (VarSetIsIn((VarSet)g->analysis, assigned))
        {
          buffer = VarSetMerge((VarSet)g->analysis, NULL);
          buffer = VarSetremoveElement(buffer, assigned);
          result = VarSetMerge(buffer, fwtools_allVariablesExpr(getSubTree(g->expr, 1)));
          VarSetDestroy(buffer);
          return result;
        }
      else
        return (Element)VarSetMerge(g->analysis, NULL);
    }
}

int slivevar_eq(Element a, Element b)
{
    return VarSetEqual((VarSet)a, (VarSet)b);
}

void slivevar_deref(Element a)
{
    VarSetDestroy((VarSet) a);
}


Element slivevar_ref(Element a)
{
  return a;
}


char *slivevar_print(Element a)
{
  return VarSetSPrint((VarSet) a);
}

int slivevar_initfunc()
{
    char buffer[35];
    int id;
    char text[512] = "Ausgabevariablen auswaehlen aus der Variablenmenge ";
    int error = 0;

    iotas = (Element)createVarSet();
    
    prompt(strcat(text, VarSetSPrint(var_all)), "0");
    
    buffer[0] = '\n';
    while ((strcmp(buffer, "0")))
    {  
	scanf("%s", &buffer);
	if (!error)
	{	
	    if (strcmp(buffer, "0"))
	    {
		toLower(buffer);
		id = SymbolTableFindSymbolByName(vartable, buffer);
		if (id != -1)
		{
		    iotas = VarSetaddElement((VarSet)iotas, id);
		}
		else 
		{
		    error = 1;
		}
	    }
	}
    }
    if (error)
    {
	livevar_finalizefunc();
	return 0;	  
    }
    else
	return 1;
}


void slivevar_finalizefunc()
{
  /* freeing the iotas element */
  VarSetDestroy(iotas);
}


FrameWork fw_slivevar_createFrameWork(int direction)
{
    FrameWork result = (FrameWork)malloc(sizeof(struct FrameWork_));
    result->direction = direction;
    result->iota = (Iota)&slivevar_iota;
    result->supremum = (Supremum)&slivevar_supremum;
    result->bottom = (Bottom)&slivevar_bottom;
    result->transfer = (Transfer)&slivevar_transfer;
    result->eq = (Eq)&slivevar_eq;
    result->ref = (RefElement)&slivevar_ref;
    result->deref = (DeRefElement)&slivevar_deref;
    result->printfunc = (PrintFunc)&slivevar_print;
    result->initfunc = (InitFunc)&slivevar_initfunc; 
    result->finalizefunc = (FinalizeFunc)&slivevar_finalizefunc;

    return result;
}
