/*
fwtools.c
12.06.2004

Provides functions for building frameworks

*/

#include "fwtools.h"
#include "parsetree.h"
#include "symbols.h"


VarSet var_all;
extern SymbolTable vartable;

/* important: this procedure has to be called AFTER building the 
FlowGraph
*/
/*
  void fwtools_init()
  initializes global variables etc.
  it is necessary to call this function BEFORE
  using the other functions of fwtools
*/
void fwtools_init()
{
  var_all = fwtools_allVars();
}


/*
  int fwtools_assignedVariable(ParseTree t)
  if the given ParseTree t represents an assignment,
  the id of the assigned variable will be returned;
  else -1
*/
int fwtools_assignedVariable(ParseTree t)
{
    if (t->type == ASSIGN)
	return getSubTree(t,0)->tag.value;
    else
	return -1;
}

/*
  int fwtools_VariableinExpression(int var, ParseTree t)
  returns:
  1 if the given variable var is part of the ParseTree t
  0 else
*/
int fwtools_VariableinExpression(int var, ParseTree t)
{
    /* recursive search for variables */
    switch (t->type)
    {
    case ADD:
      if (fwtools_VariableinExpression(var, getSubTree(t, 0)))
	return 1;
      else
	return fwtools_VariableinExpression(var, getSubTree(t, 1));
    case SUB:
      if (fwtools_VariableinExpression(var, getSubTree(t, 0)))
	return 1;
      else
	return fwtools_VariableinExpression(var, getSubTree(t, 1));
    case MUL:
      if (fwtools_VariableinExpression(var, getSubTree(t, 0)))
	return 1;
      else
	return fwtools_VariableinExpression(var, getSubTree(t, 1));
    case DIV:
      if (fwtools_VariableinExpression(var, getSubTree(t, 0)))
        return 1;
      else
	return fwtools_VariableinExpression(var, getSubTree(t, 1));
    case VARIABLE:
	    return (t->tag.value == var);
    	
	default: return 0;
    }   
}


/*
  VarSet fwtools_allVariablesExpr(ParseTree t);
  Returns an VarSet which includes all variables of t
*/
VarSet fwtools_allVariablesExpr(ParseTree t)
{
  VarSet result = createVarSet();
  VarSet b1, b2;
  if (t == NULL) return NULL;
  if ((t->type == ADD) || (t->type == SUB) || (t->type == MUL) || (t->type == DIV) ||
      (t->type == EQUAL) || (t->type == NOTEQUAL) || (t->type == SMALLEREQUAL) ||
      (t->type == GREATEREQUAL) || (t->type == SMALLER) || (t->type == GREATER) ||
      (t->type == AND) || (t->type == OR) || (t->type == MOD))
    {
      /* binary operators / relations */
      b1 = fwtools_allVariablesExpr(getSubTree(t, 0));
      b2 = fwtools_allVariablesExpr(getSubTree(t, 1));
      result = VarSetMerge(b1, b2);
      VarSetDestroy(b1); 
      VarSetDestroy(b2);
      return result;
    }
  if (t->type == NOT)
    {
      /* unary operators / relations*/
      return fwtools_allVariablesExpr(getSubTree(t, 0));
    }
  else if (t->type == VARIABLE)
      return VarSetaddElement((VarSet)NULL, t->tag.value);
  else if (t->type == INTEGER)
        return NULL;
  else /* error */
      return NULL;
}


/*
  VarSet fwtools_allVars()
  initializes the global variable var_all (list of all variables
  in the current program)
  is used in fwtools_init()
*/
VarSet fwtools_allVars()
{
    SymbolCell walker = vartable->entries;
    VarSet result = createVarSet();
    while (walker != NULL)
    {
	/* add each variable to the list */
	result = VarSetaddElement(result, walker->id);
	walker = walker->next;
    }
    return result;
}
