/* 
   guictrl.c
   Interface for communicating with the
   GUI

*/

#include "guictrl.h"
#include "string.h"
#include "deleting.h"
#include "flowrel_list.h"

extern ParseTree parsetree;
extern FlowGraph graph;
extern FILE *yyin;
extern int yydebug;
extern int parsing_ok;
extern GenericList fwlist;
extern SymbolTable vartable;
extern int nrcommand;
extern VarSet var_all;
extern FrameWork actual_fw;
extern FlowRelationList worklist;
extern LabelList labellist;
extern void yy_new_file();
extern int yydebug;
extern char errtoken[128];
extern int row;

/* Output procedures for the graphical interface (dot-files) */


void outputFlowGraph(FlowGraph g, FrameWork fw)
{
    FlowRelationElement first = FlowRelationListFirst(worklist);
    FlowNodeList w = g->all, h;
    char *ana1, *ana2;
    Element t_ana;
    /* printList(worklist); */
    printf("digraph FlowGraph {\n");
    /* format definitions */
    printf("node [shape=box, style=rounded]\n");
    while (w != NULL)
    {
	t_ana = fw->ref((Element)(fw->transfer)(w->node, w->node->analysis));
	ana1 = fw->printfunc(w->node->analysis);
	ana2 = fw->printfunc(t_ana);
	fw->deref(t_ana);
        /*printf("node%d [label=\"%s (%d) | %s | %s\"]\n",
	       w->node->label, 
	       nn(findLabelbyID(labellist, w->node->expr->label)), 
	       w->node->label,
	       ana1,
	       ana2);
	*/
	printf("node%d [label=\"%s | ", w->node->label,nn(findLabelbyID(labellist, w->node->expr->label))  );
	inOrderCommand(w->node->expr);
	printf(" | %s | %s\"]\n", ana1,
	      ana2);
	free(ana1);
	free(ana2);
	w = w->next;
    }

    /* inserting the edges */

    w = g->all;
    while (w != NULL)
    {	
	h = w->node->exit;
	while (h != NULL)
	{
          printf("node%d->node%d [dir=none",
		   w->node->expr->label,
		   h->node->expr->label
		   );
	  
          if (((fw->direction != 1) && (first.from == w->node && first.to == h->node))
	      ||
	      ((fw->direction == 1) && (first.to == w->node && first.from == h->node)))
            printf(", color=\"red\"");
	  else 
	  {
            FlowRelationList l;
            for (l = worklist; l; l = l->next)
		if (fw->direction == 1)
		{
		if (l->value.to == w->node && l->value.from == h->node)
		    printf(", color=\"blue\"");
		}else
		{
		if (l->value.from == w->node && l->value.to == h->node)
		    printf(", color=\"blue\"");
		}
	  }
          printf("]\n");
          h = h->next;
	}
	w = w->next;
    }
    printf("}\n\n");
}


void outputFlowGraph_simple(FlowGraph g)
{
    FlowNodeList w = g->all, h;
    
    printf("digraph FlowGraph {\n");
    /* format definitions */
    printf("node [shape=box, style=rounded]\n");
    while (w != NULL)
    {
	printf("node%d [label=\"%s | ", w->node->label,nn(findLabelbyID(labellist, w->node->expr->label))  );
	inOrderCommand(w->node->expr);
	printf("  \"]\n");

	/*printf("node%d [label=\"%s (%d) |  | \"]\n", 
             w->node->label, 
             nn(findLabelbyID(labellist, w->node->expr->label)),
             w->node->label
             );*/
      w = w->next;
    }
    
    /* inserting the edges */
    
    w = g->all;
    while (w != NULL)
    {	
	h = w->node->exit;
	while (h != NULL)
	{
          printf("node%d->node%d [dir=none]\n",
                 w->node->expr->label,
                 h->node->expr->label
                 );
	    h = h->next;
	}
	w = w->next;
    }
    printf("}\n\n");
}
	


/*
  void newSource(char *filename)
  Tries to open and parse the file with the given filename,
  and builds the FlowGraph
*/
int newSource(char *filename)
{    
    row = 1;
    nrcommand = 0;
    parsing_ok = 1;
    vartable = createSymbolTable();
    labellist = createLabelList();   
    yyin = fopen(filename, "r");
    yy_new_file();
    if (yyin == NULL) 
      {
        printf("ERROR\n Datei nicht gefunden: <%s>\n", filename);
        return -1;
      }
    yydebug = 1;
    
    yyparse(); 
    fclose(yyin);
    
    if (parsing_ok == 1)
    {
	graph = generateFlowGraph(parsetree);
	return 1;
    }
    else
      return -1;
}

/*
  void printAnalysisList()
  Prints the descriptions of the analysis list to stdout
*/
void printAnalysisList()
{
    char *buffer;
    GenericCell walker = fwlist->first;
    while (walker != NULL)
    {
	printf("%s\n", ((FrameWorkEntry)(walker->value))->name);
	walker = walker->next;      
    }
    printf("\n");  /* final newline */
}


int guicontrol()
{
    char command[256];
    int z;
    /* free_elements determines if it's necessary to
       destroy the analysis results of the flow graph */
    int free_elements = 0;
    
    setbuf(stdin,NULL);
    setbuf(stdout,NULL);
    if (scanf("%s", command) == EOF) return 1;
    
    graph = NULL;
    actual_fw = NULL;
    parsetree = NULL;
    worklist = NULL;

    init_fwlist();

    while (strcmp(":exit", command) != 0)
    {
      if (strcmp(":frameworks", command) == 0)
	{
          printAnalysisList();
	}
      
      else if(strcmp(":analysis", command) == 0)
	{ 

          scanf("%s", command); 

	  if (graph == NULL)
            {
              /* no file opened so far*/
              printf("ERROR\nNoch keine Datei ausgewaehlt! \n");
            }
          else
            {
              if (getActualFrameWork() != NULL)
                {
                  /* Some cleaning up to do ... */
                  /* we can keep the FlowGraph itself, but
                     we have to delete the analysis results*/
		 
		    if ((graph != NULL) && (free_elements))
			deleteFlowGraphElements(graph, getActualFrameWork());
		
		    getActualFrameWork()->finalizefunc();
		
		}
	      
              setActualFrameWork(atoi(command));  
	   
              if (initWorkList(graph, getActualFrameWork()))
	      {
		  outputFlowGraph(graph, getActualFrameWork());
	      }
	      else
	      {
		  printf("ERROR\n Fehler bei der Instanziierung des Frameworks\n");
		  setActualFrameWork(-1);
	      } 
	      
	      free_elements = 1;
            }
	}
      
      else if(strcmp(":filename", command) == 0)
	{
          scanf("%s", command);
	  
	  deleteParseTree(parsetree);  
	  deleteSymbolTable(vartable);
	  deleteLabelList(labellist);


	  if ((getActualFrameWork() != NULL))
          {
	      deleteFlowGraph(graph, getActualFrameWork(), free_elements);
	  }
      
	  graph = NULL;
          parsetree = NULL;
          vartable = NULL;
          labellist = NULL;       

	  if (newSource(command) != -1)
	  {
	      if (getActualFrameWork() != NULL)
	      {
		  VarSetDestroy(var_all);
                  var_all = NULL;
	      }
	      fwtools_init();
	      outputFlowGraph_simple(graph);
	  }
	  else
	  {
	      printf("ERROR\n Fehler beim Parsen, Zeile %d bei \"%s\"\n", row, errtoken);
	  }
	  free_elements = 0;
	}	
      
      else if (strcmp(":next", command) == 0)
	{
          if ((getActualFrameWork()) == NULL)
            {
              printf("ERROR\n Kein Framework ausgewaehlt\n");
              
            }
          else
            {
          if (stepWorkList(graph, getActualFrameWork()) == 1)
            printf("READY\n");
          else
            outputFlowGraph(graph, getActualFrameWork());
            }
        }
      
        if (scanf("%s", command) == EOF)  return 1;
    }
    
    return 1;
}

