#! /usr/bin/perl -w

###############################################################################

require 5.002;

use English;
use Tk;
use Tk::Dialog;
use Tk::ErrorDialog;
use Tk::FileSelect;
use strict;

###############################################################################
# Global subroutine and variable declarations required by my() and use strict.
###############################################################################

use subs qw(handler dialogError entryFileLoad);

my ($VIEW,$VIEW_TEXT);
my $VERSION = "0.1";

###############################################################################
# Create and initialize main window.
###############################################################################
my $MW = MainWindow->new;
$MW->title("GUI fuer wAnalyzer $VERSION (nichts geladen)");
$MW->iconname('WA');

my $WLA_PROGNAME = "./wanalyzer -gui";
my $DOT_PROGNAME = "dot -Tplain";

my $FONT = '-*-Helvetica-Medium-R-Normal--*-140-*-*-*-*-*-*';
my $CFONT = '-*-Helvetica-Bold-R-Normal--*-140-*-*-*-*-*-*';
my $CANVAS_HEIGHT = "20c";
my $DRAWID = 1;
my $command =  sub { print TO_WLA_WRITE ":exit\n"; exit(0); };
$MW->bind('<Control-q>' => $command);
$MW->bind('<Meta-q>' => $command);

###############################################################################
# Populate the main window with a menu bar and a canvas.
###############################################################################

###############################################################################
# Create the menu bar.
###############################################################################

my $menuBar = $MW->Frame;
$menuBar->pack(-side => 'top', -fill => 'x');

# Create a button to load a file of statements.
# This button will be configured after the canvas is created.
my $menuBar_load = $menuBar->Button(
    -text      => 'Laden',
    -underline => 0,
    -bd        => '0',
);
$menuBar_load->pack(-side => 'left');

###############################################################################
# Create the drawing canvas.
###############################################################################

my $w_frame = $MW->Frame();
$w_frame->pack(-side => 'top', -fill => 'both', -expand => 'yes');

my $c = $w_frame->Canvas(
    -scrollregion => ['0c', '0c', '8.5i', '11i'],
    -width        => '15c',
    -height       => $CANVAS_HEIGHT,
    -relief       => 'sunken',
    -bd           => 2,
);
$command = sub {entryFileLoad($c);};
$menuBar_load->configure(-command => $command);
$MW->bind('<Alt-l>' => $command);
$MW->bind('<Meta-l>' => $command);

# Add vertical and horizontal scrollbars to the canvas frame.

my $w_frame_vscroll = $w_frame->Scrollbar(-command => [$c => 'yview']);
my $w_frame_hscroll = $w_frame->Scrollbar(
    -orient => 'horiz',
    -command => [$c => 'xview'],
);
$c->configure(-xscrollcommand => [$w_frame_hscroll => 'set']);
$c->configure(-yscrollcommand => [$w_frame_vscroll => 'set']);
$w_frame_hscroll->pack(-side => 'bottom', -fill => 'x');
$w_frame_vscroll->pack(-side => 'right', -fill => 'y');
$c->pack(-expand => 'yes', -fill => 'both');

# Choose fonts and colors.

# my $font1 = '-*-Helvetica-Medium-R-Normal--*-120-*-*-*-*-*-*';
# my $font2 = '-*-Helvetica-Bold-R-Normal--*-240-*-*-*-*-*-*';
my($blue, $red, $bisque, $green);
if ($MW->depth > 1) {
    $blue = 'DeepSkyBlue3';
    $red = 'red';
    $bisque = 'bisque3';
    $green = 'SeaGreen3';
} else {
    $blue = 'black';
    $red = 'black';
    $bisque = 'black';
    $green = 'black';
}

my $w_stmt = $MW->Frame;
my $w_stmt_next  = $w_stmt->Button(-text => 'Next');
$w_stmt_next->pack(-side => 'right', -pady => 5, -padx => 10);
$w_stmt->pack(-side => 'top', -fill => 'x');

$w_stmt_next->configure(-command => [\&next_step,$c]);


###############################################################################
# Start the parser.
###############################################################################

# Install signal handler to detect broken pipes.
$SIG{'PIPE'} = \&handler;

# Create two pipes to establish bidirectional communication to/from ddcal.
pipe(TO_WLA_READ,TO_WLA_WRITE);
pipe(FROM_WLA_READ,FROM_WLA_WRITE);

# Unbuffer TO_WLA_WRITE.
select((select(TO_WLA_WRITE), $| = 1)[$[]);

# Fork the analyzer.

my $pid;

FORK_WLA: {
    if ($pid = fork) { # parent
	close(TO_WLA_READ);
	close(FROM_WLA_WRITE);
    } elsif (defined $pid) { # child
	open(STDIN, "<&TO_WLA_READ") || die "can't dup to stdin";
	open(STDOUT, ">&FROM_WLA_WRITE") || die "can't dup to stdout";
	close(TO_WLA_WRITE);
	close(FROM_WLA_READ);
	close(TO_WLA_READ);
	close(FROM_WLA_WRITE);
	exec("$WLA_PROGNAME") || die "Can't exec $WLA_PROGNAME: $!\n";
    } elsif ($! =~ /No more process/) { # try again later
	sleep 5;
	redo FORK_WLA;
    } else {	# can't figure it out: give up
	die "Can't fork: $!\n";
    }
}

###############################################################################
# Start dot
###############################################################################

# Create two pipes to establish bidirectional communication to/from ddcal.
pipe(TO_DOT_READ,TO_DOT_WRITE);
pipe(FROM_DOT_READ,FROM_DOT_WRITE);

# Unbuffer TO_WLA_WRITE.
select((select(TO_DOT_WRITE), $| = 1)[$[]);

open STDERR,">&TO_WLA_WRITE";
# Fork the analyzer.

FORK_DOT: {
    if ($pid = fork) { # parent
	close(TO_DOT_READ);
	close(FROM_DOT_WRITE);
    } elsif (defined $pid) { # child
	open(STDIN, "<&TO_DOT_READ") || die "can't dup to stdin";
	open(STDOUT, ">&FROM_DOT_WRITE") || die "can't dup to stdout";
	close(TO_DOT_WRITE);
	close(FROM_DOT_READ);
	close(TO_DOT_READ);
	close(FROM_DOT_WRITE);
	exec("$DOT_PROGNAME") || die "Can't exec $DOT_PROGNAME: $!\n";
    } elsif ($! =~ /No more process/) { # try again later
	sleep 5;
	redo FORK_DOT;
    } else {	# can't figure it out: give up
	die "Can't fork: $!\n";
    }
}

###############################################################################

# Get the list of frameworks.
my @analyses = ();

my $menuBar_fw = $menuBar->Menubutton(
    -text      => 'Frameworks',
    -underline => 0,
);

print TO_WLA_WRITE ":frameworks\n";
my $thisline = "";
while (defined(\*FROM_WLA_READ) && ($thisline = <FROM_WLA_READ>)) {
	chop($thisline);
	last if $thisline eq "";
	push @analyses,$thisline;

	$menuBar_fw->command(
	    -label     => "$thisline",
	    -underline => 0,
	    -command   => [\&start_analysis, $c, $#analyses]
	);
	
}
$menuBar_fw->pack(-side => 'left');


###############################################################################
# Enter the event-processing loop.
###############################################################################

MainLoop;

# Not reached.


##Sub##########################################################################

sub start_analysis {
    my ($canvas,$num) = @_;

    print TO_WLA_WRITE ":analysis\n$num\n";
    process_graph($canvas);
}

##Sub##########################################################################

sub next_step {
    my ($canvas) = @_;

    print TO_WLA_WRITE ":next\n";
    process_graph($canvas);
}

##Sub##########################################################################
#
# Synopsis	[Processes a statement and draws the BDD on the canvas.]
#
# Description	[Processes a statement and draws the BDD on the canvas.
#		The input statement is sent to the parser. The parser
#		responds in one of three ways: (1) By sending a graph
#		for a BDD. In this case the first line starts with
#		"graph" and the last line is "stop". (2) By sending just
#		the word "stop". This happens when no drawing is
#		generated by the input statement. (3) By sending just
#		the word "error". This happens when the input statement
#		contains a syntax error. This procedure parses the
#		response of the parser and, if required, draws a new BDD
#		on the canvas.]
#
# SideEffects	[]
#
###############################################################################
sub process_graph {

    my ($c) = @ARG;
    my $savecursor = $MW->cget(-cursor);
    $MW->configure(-cursor => 'watch');
    # Deal with continuation lines. A backslash in a comment
    # does not indicate continuation. Therefore a backslash must not be
    # preceded by '#' and may only be followed by white space.

    my $thisline;
    my ($name,$label);
    my ($graph, $scalefactor, $bounding_box_x, $bounding_box_y);
    my $offset_x = 10;
    my $offset_y = 10;
    my ($bbulx,$bbuly,$bblrx,$bblry);

    while (defined(\*FROM_WLA_READ) && ($thisline = <FROM_WLA_READ>)) {
#	print STDERR "GUI: $thisline";
	if ($thisline eq "ERROR\n") {
	    $thisline = <FROM_WLA_READ>;
	    $MW->configure(-cursor => $savecursor);
	    dialogError($thisline);
	    $MW->title("GUI fuer wAnalyzer $VERSION (Fehler)");
	    return;
	}
	if ($thisline eq "READY\n") {
	    $MW->configure(-cursor => $savecursor);
	    return;
	}
	if ($thisline eq "PROMPT\n") {
		my $prompt = <FROM_WLA_READ>;
		my $final = <FROM_WLA_READ>;
		$MW->configure(-cursor => $savecursor);
		getPromptFromUser($c,$prompt,$final);
		return;
	}
	last if $thisline eq "\n";
	print TO_DOT_WRITE $thisline;
    }

    while (defined(\*FROM_DOT_READ) && ($thisline = <FROM_DOT_READ>)) {
	chop($thisline);
	if ($thisline =~ /^graph/) {
	    $c->delete('all');
	    ($graph, $scalefactor, $bounding_box_x, $bounding_box_y) = split(' ',$thisline);
	    $bounding_box_x *= 72;
	    $bounding_box_y *= 72;
	} elsif ($thisline =~ /^node/) {
	    $thisline =~ s/^node\s+//;
	    if ($thisline =~ /^(\"[^\"]*\")/) {
		$name = $1;
	    } elsif ($thisline =~ /(\S+)/) {
		$name = $1;
	    } else {
		$name = "unknown";
	    }
	    $thisline =~ s/^$name\s+//;
	    $thisline =~ /((\d+\.\d+)\s+(\d+\.\d+)\s+(\d+\.\d+)\s+(\d+\.\d+))/;
	    my $numbers = $1;
	    my $center_x = $2;
	    my $center_y = $3;
	    my $xsize = $4;
	    my $ysize = $5;
	    $thisline =~ s/^$numbers\s+//;
	    if ($thisline =~ /^(\"[^\"]*\")/) {
		$label = $1;
	    } elsif ($thisline =~ /(\S+)/) {
		$label = $1;
	    } else {
		$label = "unknown";
	    }
	    $thisline =~ s/^$label\s+//;
	    my $style;
	    if ($thisline =~ /\b(invis)\b/) {
		next;
	    }
	    if ($thisline =~ /\b(plaintext)\b/) {
		$style = $1;
	    } elsif ($thisline =~ /\b(ellipse)\b/) {
		$style = $1;
	    } elsif ($thisline =~ /\b(box)\b/) {
		$style = $1;
	    } else {
		$style = "unknown";
	    }
	    $label =~ tr/\"//d;
	    # print "node $label $center_x $center_y $xsize $ysize $style\n";
	    # Convert from inches to points.
	    $center_x = $offset_x + $center_x * 72;
	    $center_y = $offset_y + $bounding_box_y - $center_y * 72;
	    $xsize *= 72;
	    $ysize *= 72;
	    # Convert from center/size to upper-left/lower-right.
	    $bbulx = $center_x - $xsize / 2.0;
	    $bbuly = $center_y - $ysize / 2.0;
	    $bblrx = $center_x + $xsize / 2.0;
	    $bblry = $center_y + $ysize / 2.0;
	    # Apply the scaling factor
	    $center_x *= $scalefactor;
	    $center_y *= $scalefactor;
	    $bbulx *= $scalefactor;
	    $bbuly *= $scalefactor;
	    $bblrx *= $scalefactor;
	    $bblry *= $scalefactor;
	    # print "node $label $bbulx $bbuly $bblrx $bblry $style $center_x $center_y\n";
	    if ($style eq "ellipse") {
		$c->create('oval',$bbulx,$bbuly,$bblrx,$bblry,
			   -outline => $blue, qw(-width 2 -tags item));
	    } elsif ($style eq "box") {
		$c->create('rect',$bbulx,$bbuly,$bblrx,$bblry,
			   -outline => $blue, qw(-width 2 -tags item));
	    } elsif ($style eq "plaintext") {
	    } elsif ($style eq "unknown") {
	    }
	    if ($DRAWID || $style ne "ellipse") {
		$c->create('text',$center_x,$center_y,'-text',
			    $label,'-anchor','c','-font',$CFONT);
	    }
	} elsif ($thisline =~ /^edge/) {
	    $thisline =~ s/^edge\s+//;
	    if ($thisline =~ /^(\"[^\"]*\")/) {
		$name = $1;
	    } elsif ($thisline =~ /(\S+)/) {
		$name = $1;
	    }
	    $thisline =~ s/^$name\s+//;
	    if ($thisline =~ /^(\"[^\"]*\")/) {
		$name = $1;
	    } elsif ($thisline =~ /(\S+)/) {
		$name = $1;
	    }
	    $thisline =~ s/^$name\s+//;
	    $thisline =~ /(\d+)/;
	    my $points = $1;
	    $thisline =~ s/^$points\s+//;
	    my @points;
	    @points = ();
	    my ($thispoint,$thisx,$thisy);
	    for (1 .. $points) {
		$thisline =~ /((\d+\.\d+)\s+(\d+\.\d+))/;
		$thispoint = $1;
		$thisx = $2;
		$thisy = $3;
		$thisx = $thisx * 72 + $offset_x;
		$thisy = $bounding_box_y - $thisy * 72 + $offset_y;
		# Apply scaling factor.
		$thisx *= $scalefactor;
		$thisy *= $scalefactor;
		push(@points,$thisx);
		push(@points,$thisy);
		$thisline =~ s/^$thispoint\s+//;
	    }
	    my $style;
	    if ($thisline =~ /\b(invis)\b/) {
		next;
	    }
	    if ($thisline =~ /\b(solid)\b/) {
		$style = $1;
	    } elsif ($thisline =~ /\b(dashed)\b/) {
		$style = $1;
	    } elsif ($thisline =~ /\b(dotted)\b/) {
		$style = $1;
	    } else {
		$style = "unknown";
	    }
	    my ($colour) = ($thisline =~ /\b(\w*)$/);
	    # print "edge $points @points $style\n"; # DEBUG
	    $c->create('line',@points,'-smooth','on',
		   -fill => $colour, qw(-width 2 -tags item -arrow last));
	} elsif ($thisline =~ /^stop/) {
	    last;
	} elsif ($thisline =~ /^beep/) {
	    $MW->bell;
	    last;
	} elsif ($thisline =~ /^error/) {
	    $MW->bell;
	    &dialogError("Error while processing the graph.");
	    last;
	}
    }
    $MW->configure(-cursor => $savecursor);

} # end process_statement


##Sub##########################################################################
#
# Synopsis	[Generic signal handler.]
#
# Description	[Generic signal handler. Reports what signal has been
#		caught by creating a dialog box.]
#
# SideEffects	[None.]
#
###############################################################################
sub handler {
    my ($sig) = @_;
    my $dialog;
    my($ok) = ('OK');

    $dialog = $MW->Dialog(
	-title          => 'Signal Received',
	-text           => '',
	-bitmap         => 'error',
	-default_button => $ok,
	-buttons        => [$ok],
    );
    $dialog->configure(
	-wraplength => '4i',
	-text       => "Received a SIG$sig signal. Communications with the computation engine may by interrupted. Click on the \"OK\" button to continue.",
    );

    $dialog->Show;

} # end handler


##Sub##########################################################################
#
# Synopsis	[Displays dialog box for syntax errors.]
#
# Description	[]
#
# SideEffects	[None.]
#
###############################################################################
sub dialogError {

    my ($text) = @_;
    my $dialog;
    my($ok) = ('OK');

    $dialog = $MW->Dialog(
	-title          => 'Error',
	-text           => '',
	-bitmap         => 'error',
	-default_button => $ok,
	-buttons        => [$ok],
    );
    $dialog->configure(
	-wraplength => '4i',
	-text       => $text
    );

    $dialog->Show;

} # end dialogError


##Sub##########################################################################
#
# Synopsis	[Displays selection box for file loading.]
#
# Description	[]
#
# SideEffects	[None.]
#
###############################################################################
sub entryFileLoad {

    my ($canvas) = @ARG;
    my $start_dir = ".";
    my $FSref = $MW->FileSelect(-directory => $start_dir);
    $FSref->title('Load File');
    $FSref->iconname('Load File');
    my $filename = $FSref->Show;
    return unless defined($filename);

    if (!-r $filename) {
	$MW->Dialog(
	    -title  => 'File Not Found',
	    -text   => $OS_ERROR,
	    -bitmap => 'error',
	)->Show;
	return;
    }
    print TO_WLA_WRITE ":filename\n$filename\n";
    process_graph($canvas);
    $MW->title("GUI fuer wAnalyzer $VERSION ($filename)");

} # end readFile


###############################################################################

sub getPromptFromUser {

    my ($canvas,$prompt,$final) = @ARG;
    my $w;

    $w = $MW->Toplevel;
    $w->geometry('+400+400');
    $w->title('Anfangswert setzen');
    $w->iconname('Anfangswert');
    my $w_msg = $w->Label(
        -wraplength => '4i',
        -justify    => 'left',
        -text       => $prompt
    );
    $w_msg->pack;

    my $answer = "";
    my $w_e = $w->Entry(-relief => 'sunken', -textvariable => \$answer);
    my @pl = (-side => 'top', -padx => 10, -pady => 5, -fill => 'x');
    $w_e->focus;
    $w_e->pack(@pl);

    my $w_buttons = $w->Frame;
    $w_buttons->pack(qw(-side bottom -fill x -pady 2m));
    my $command = sub {print TO_WLA_WRITE "$answer $final";
                       $w->destroy;
			process_graph($canvas);
                   };
    my $w_save = $w_buttons->Button(
        -text    => 'Ok',
        -command => $command,
    );
    $w_save->pack(qw(-side left -expand 1));

}


###############################################################################
# Give usage information.
###############################################################################
sub usage {
    print <<"USAGE"; exit 0;
Usage: $0
USAGE

} # end usage

1;
