
%{
  #include <string.h>
  #include <stdio.h>
  #include <stdlib.h>
  #include "symboltable.h"/* includes the SymbolTable structure */
  #include "parsetree.h"  /* includes the ParseTree structure */
  #include "symbols.h"    /* Token identifiers */
  char *identptr;
  int varid;
  int row = 1;
  SymbolTable vartable = NULL;
  /* Label identifiers are stored in the labeltable
     Every command/boolean expression has his own label (they are
     enumerated, beginning by 0), but the user can specify a label identifier
     for a command, simply by putting it at the beginning of the line:
     @Label1: skip
     for example. The system label (id) is stored in the ParseTree structure. 
 */
 
   char errtoken[128];
  int lexing_ok;

void yy_new_file ()
{
  YY_NEW_FILE;
}
%}


%%


\n                {row++;}
[ \t]+            {/**/}
\-\-[^\n]*        {/* ignore comments and whitespaces */}

\=    {return EQUAL;}
\<\>  {return NOTEQUAL;}
\<    {return SMALLER;}
\<\=  {return SMALLEREQUAL;}
\>    {return GREATER;}
\>\=  {return GREATEREQUAL;}

[Tt][Rr][Uu][Ee]  {yylval.tree = createParseTree(TRUE);
                       return TRUE;}
[Ff][Aa][Ll][Ss][Ee] {yylval.tree = createParseTree(FALSE);
                           return FALSE;}
[Nn][Oo][Tt]   {return NOT;}
[Aa][Nn][Dd]   {return AND;}
[Oo][rR]    {return OR;}

\-    {return MINUS;}
\+    {return PLUS;}
\*    {return MUL;}
\/    {return DIV;}
[Mm][Oo][Dd]   {return MOD;}
\^    {return POT;}
[Ii][fF]    {return IF;}
[Tt][Hh][Ee][Nn]  {return THEN;}
[Ee][Ll][Ss][Ee]  {return ELSE;}
[Ff][Ii]    {return FI;}
[Ww][Hh][Ii][Ll][Ee] {return WHILE;}
[Dd][Oo]    {return DO;}
[Oo][Dd]    {return OD;}
[Ss][Kk][Ii][Pp]  {return SKIP;}
":="  {return ASSIGN;}
\;    {return SEMICOLON;}
\(    {return LPAR;}
\)    {return RPAR;}

[P|p][U|u][T|t]          {return PUT;}
[G|g][E|e][T|t]          {return GET;}

[0-9]+                    { yylval.tree = createParseTree(INTEGER);
                            setVarId(yylval.tree, atoi(yytext)); 
                            return INTEGER;}

[a-zA-Z][a-zA-Z0-9\_]*     { if ((identptr = (char*)(malloc(sizeof(char)*(strlen(yytext)+1)))) == NULL)
                              return ERROR;
                            else
                              {
                                toLower(yytext);
                                varid =  SymbolTableAddSymbol(vartable, strncpy(identptr, yytext, strlen(yytext)+1)); 
                                yylval.tree = createParseTree(VARIABLE);
                                setVarId(yylval.tree, varid);
                                return VARIABLE;
                              }
                           }
@[a-zA-Z0-9][a-zA-Z0-9\_]*\:    { 
                                  yylval.label = strncpy(malloc(sizeof(char)*(strlen(yytext))), yytext+1, strlen(yytext)-2);
                                  yylval.label[strlen(yytext)-2] = '\0';
                                  return LABEL; 
                                 }


.                          {/*printf("ERROR\n<%s> in Zeile %d\n", yytext, row);*/
                            strcpy(errtoken, yytext);
                            lexing_ok = 0; return ERROR;}
%%
