
%{
#include "parsetree.h"
#include "symboltable.h"
#include "parsercontrol.h"
#include <stdio.h>
#include "labellist.h"
#include "framework.h"
 ParseTree parsetree = NULL;
 extern char * yytext;
 int nrcommand = 0;
 extern SymbolTable vartable;
 extern int row;
 LabelList labellist = NULL;
 int parsing_ok = 1;
 extern char errtoken[128];
 extern int lexing_ok;
%}


%token <tree> INTEGER VARIABLE TRUE FALSE
%token <tree> IF THEN ELSE FI WHILE DO OD SKIP ASSIGN ENDSTATEMENT
%token <tree> EQUAL NOTEQUAL SMALLEREQUAL GREATEREQUAL SMALLER GREATER
%token <tree> NOT AND OR
%token <tree> SEMICOLON LPAR RPAR 
%token <tree> NEG PLUS MINUS MUL DIV MOD POT 
%token <tree> ERROR SEQ
%token <tree> GET PUT
%token <label> LABEL
%token <tree> ADD SUB UMINUS
%type  <tree> PROG SIMPLESTATEMENT STATEMENT SKIPSTATEMENT IFSTATEMENT WHILESTATEMENT ASSIGNSTATEMENT
%type  <tree> BOOLEXPR RELEXPR ARITHEXPR IOSTATEMENT


%left OR
%left AND
%nonassoc NOT

%left PLUS MINUS
%left MUL DIV MOD
%left POT
%nonassoc UMINUS


%%


PROG: STATEMENT SEMICOLON PROG
   {
    if ($3->type == SEQ)
    {
	$$ = $3;
	addSubTree($$, $1, FIRST); 
	parsetree = $$;
     }
     else
     {
	$$ = createParseTree(SEQ);
	addSubTree($$, $1, FIRST);
	addSubTree($$, $3, LAST);
	parsetree = $$;
     }
    
    }
    | STATEMENT    {$$ = $1; parsetree = $1; }
;

STATEMENT: LABEL SIMPLESTATEMENT {nrcommand++; $$ = $2; $$->label = nrcommand; labellist = addLabel(labellist, nrcommand, $1);}
| SIMPLESTATEMENT {nrcommand++; $$ = $1; $$->label = nrcommand;}
;

SIMPLESTATEMENT:
           SKIPSTATEMENT    {$$ = $1;}
         | IFSTATEMENT      {$$ = $1;} 
         | WHILESTATEMENT   {$$ = $1;} 
         | ASSIGNSTATEMENT  {$$ = $1;}
	 | IOSTATEMENT	    {$$ = $1;}
;
 

SKIPSTATEMENT: SKIP {$$ = createParseTree(SKIP);}
;

IFSTATEMENT: IF BOOLEXPR THEN PROG ELSE PROG FI 
             {$$ = createParseTree(IF);
	      addSubTree($$, $2, LAST);
	      addSubTree($$, $4, LAST);
	      addSubTree($$, $6, LAST);
             }	      
; 

WHILESTATEMENT: WHILE BOOLEXPR DO PROG OD
		{$$ = createParseTree(WHILE);
		 addSubTree($$, $2, LAST);
		 addSubTree($$, $4, LAST);     
		}
;

ASSIGNSTATEMENT: VARIABLE ASSIGN ARITHEXPR
		 {$$ = createParseTree(ASSIGN);
		  addSubTree($$, $1, LAST);
		  addSubTree($$, $3, LAST);
		 } 
; 
		 

IOSTATEMENT:	 GET LPAR VARIABLE RPAR
		{$$ = createParseTree(GET);
		 addSubTree($$, $3, LAST);
		}
	       | PUT LPAR ARITHEXPR RPAR
	        {$$ = createParseTree(PUT);
		 addSubTree($$, $3, LAST);
		}
;

BOOLEXPR:   TRUE                     {$$ = $1;}
	  | FALSE                    {$$ = $1;}
	  | RELEXPR                  {$$ = $1;}   
          | BOOLEXPR AND BOOLEXPR    
	    {$$ = createParseTree(AND);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
          | BOOLEXPR OR BOOLEXPR
	    {$$ = createParseTree(OR);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    } 
          | NOT BOOLEXPR 
	    { $$ = createParseTree(NOT);
	      addSubTree($$, $2, LAST);
	    }
          | LPAR BOOLEXPR RPAR            {$$ = $2;} 
;

RELEXPR:   ARITHEXPR EQUAL ARITHEXPR
	    {$$ = createParseTree(EQUAL);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
         | ARITHEXPR NOTEQUAL ARITHEXPR
	    {$$ = createParseTree(NOTEQUAL);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
         | ARITHEXPR SMALLEREQUAL ARITHEXPR
	    {$$ = createParseTree(SMALLEREQUAL);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }  
         | ARITHEXPR SMALLER ARITHEXPR
	    {$$ = createParseTree(SMALLER);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }	     
         | ARITHEXPR GREATEREQUAL ARITHEXPR
	    {$$ = createParseTree(GREATEREQUAL);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }	    
         | ARITHEXPR GREATER ARITHEXPR
	    {$$ = createParseTree(GREATER);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }	    
;        

ARITHEXPR:   ARITHEXPR PLUS ARITHEXPR
	    {$$ = createParseTree(ADD);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
           | ARITHEXPR MINUS ARITHEXPR
	    {$$ = createParseTree(SUB);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
           | ARITHEXPR MUL ARITHEXPR
	    {$$ = createParseTree(MUL);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
           | ARITHEXPR DIV ARITHEXPR
	    {$$ = createParseTree(DIV);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }           
	   | ARITHEXPR MOD ARITHEXPR
	    {$$ = createParseTree(MOD);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
           | ARITHEXPR POT ARITHEXPR
	    {$$ = createParseTree(POT);
	     addSubTree($$, $1, LAST);
	     addSubTree($$, $3, LAST);
	    }
           | MINUS ARITHEXPR %prec UMINUS
	    {$$ = createParseTree(NEG);
	     addSubTree($$, $2, LAST);
	    }
           | LPAR ARITHEXPR RPAR
	    {$$ = $2;}
           | INTEGER
	    {$$ = $1;} 
           | VARIABLE
	    {$$ = $1;}
;


%%




int yyerror(char *s)
{
    if (lexing_ok)
    {
	/*printf("ERROR\n (Zeile %d): %s (Token <%s>) \n", row, s, yytext);*/
	strcpy(errtoken, yytext);
	
    }
    
    parsing_ok = 0;
}

int yywrap()
{
	return 1;	
}
