/*****************************
  
  parsercontrol.c
  
  27.03.2004
  
  Provides an interface for
  the yacc parser and the 
  parsetree
*****************************/


#include "parsetree.h"
#include "symboltable.h"
#include "flownode.h"

extern ParseTree parsetree;
extern SymbolTable vartable;
extern FILE *yyin;
extern int parsing_ok;
FlowGraph graph;
extern int yydebug;

void printFlowGraph(FlowGraph g)
{
    FlowNodeList w, w1, w2;
    w = g->all;
    while (w != NULL)
    {
      printf("Knoten: %d\n", w->node->label);
	printf( " davon Entry: ");
        w1 = w->node->entry;
        while (w1 != NULL)
          {
            printf("%d, ", w1->node->label); 
            w1 = w1->next;
          }
        printf("  davon Exit: ");
        w1 = w->node->exit;
        while (w1 != NULL)
          {
            printf("%d, ", w1->node->label); 
            w1 = w1->next;
          }
        printf("\n");
        w = w->next;
    }
    printf("------Ende\n");
}

int parseFile(char *filename)
{
    
    vartable = createSymbolTable();
    yyin = fopen(filename, "r");
    if (yyin == NULL) 
      {
        printf("Datei nicht gefunden: <%s>\n", filename);
        return 0;
      }
    /* yydebug = 1; */ 
    yyparse(); 
    fclose(yyin);
    return parsing_ok;
}


int evalBoolExpr(ParseTree tree, SymbolTable t)
{
  if (tree == NULL)
  {
    printf("\n *** ERROR: INVALID BOOLEAN EXPRESSION *** \n");
    return;
  }
  switch(tree->type)
  {
    case TRUE: return 1;
    case FALSE: return 0;
    case SMALLER: return (evalArithExpr(getSubTree(tree, 0), t) <
			  evalArithExpr(getSubTree(tree, 1), t));
    case SMALLEREQUAL: return (evalArithExpr(getSubTree(tree, 0), t) <=
			  evalArithExpr(getSubTree(tree, 1), t));
    case GREATER: return (evalArithExpr(getSubTree(tree, 0), t) >
			  evalArithExpr(getSubTree(tree, 1), t));
    case GREATEREQUAL: return (evalArithExpr(getSubTree(tree, 0), t) >=
			  evalArithExpr(getSubTree(tree, 1), t));
    case EQUAL: return (evalArithExpr(getSubTree(tree, 0), t) ==
			  evalArithExpr(getSubTree(tree, 1), t));
    case NOTEQUAL: return (evalArithExpr(getSubTree(tree, 0), t) !=
			  evalArithExpr(getSubTree(tree, 1), t));
    case AND: return (evalBoolExpr(getSubTree(tree, 0), t) &&
			  evalBoolExpr(getSubTree(tree, 1), t));
    case OR: return (evalBoolExpr(getSubTree(tree, 0), t) ||
			  evalBoolExpr(getSubTree(tree, 1), t));
    case NEG: return !evalBoolExpr(getSubTree(tree, 0), t);
    default: printf("\n *** ERROR: INVALID BOOLEAN EXPRESSION *** \n");
   }

}


int evalArithExpr(ParseTree tree, SymbolTable t)
{
  if (tree == NULL) 
  {
    printf("\n *** ERROR: INVALID ARITHMETIC EXPRESSION *** \n");
    return 0;
   }
   switch(tree->type)
   {
     case VARIABLE: return SymbolTableGetValue(t, tree->tag.value);
     case INTEGER: return tree->tag.value;
     case ADD: return(evalArithExpr(getSubTree(tree,0), t)+evalArithExpr(getSubTree(tree,1), t));
     case SUB: return(evalArithExpr(getSubTree(tree,0), t)-evalArithExpr(getSubTree(tree,1), t));
     case MUL: return(evalArithExpr(getSubTree(tree,0), t)*evalArithExpr(getSubTree(tree,1), t));
     case DIV: return(evalArithExpr(getSubTree(tree,0), t)/evalArithExpr(getSubTree(tree,1), t));
     case MOD: return(evalArithExpr(getSubTree(tree,0), t)%evalArithExpr(getSubTree(tree,1), t));
     case NEG: return (-1)*evalArithExpr(getSubTree(tree,0), t);
     default: printf("\n *** ERROR: WRONG ARITHMETIC EXPRESSION (OR NOT  IMPLEMENTED YET) *** \n");
     return 0;
   }   	
}



void execute(ParseTree tree, SymbolTable t)
{
  int getval;
  ParseTreeCell walker;
  if (tree == NULL) return;
  switch (tree->type) 
  { 
    case SKIP: return;
    case IF: if (evalBoolExpr(getSubTree(tree, 0), t))	
	       execute(getSubTree(tree, 1), t);
	     else
	       execute(getSubTree(tree, 2), t);
	     return;
    case ASSIGN: SymbolTableSetValue(t, getSubTree(tree, 0)->tag.value, 
			  evalArithExpr(getSubTree(tree, 1), t));
	         return;
    case WHILE: while (evalBoolExpr(getSubTree(tree, 0), t))
		      execute(getSubTree(tree, 1), t);
		 return;
    case GET:   printf("%s = ", SymbolTableFindSymbolById(t, getSubTree(tree,0)->tag.value));
		scanf("%d",&getval);
		SymbolTableSetValue(t, getSubTree(tree, 0)->tag.value, getval);
		return;
    case PUT:   printf(">%d\n", evalArithExpr(getSubTree(tree, 0), t));
	        return;
    case SEQ:   walker = tree->tag.subtrees->first;
		while(walker != NULL)
		{ 
		   execute(walker->value, t);
		  walker = walker->next;
		}
		return;
    default:    printf("\n *** ERROR: UNKNOWN COMMAND *** \n");
  } 	
  return;
}


void inOrder(ParseTree tree)
{
    int i;
    if (tree == NULL)
      return;
    else
    {

        if (tree->type == VARIABLE)
	{
	   printf("%s", SymbolTableFindSymbolById(vartable, tree->tag.value));
	   return;
	}
	if (tree->type == INTEGER)
	{
	   printf("%d", tree->tag.value);
	   return;
	}
	if (tree->type == SKIP)
	{
	  printf("\nSKIP");
	  return;
	}
	printf("\n\(");
        if (tree->label != -1)
          printf("%d ", tree->label);
	switch (tree->type)
	{
	 case TRUE: printf("TRUE"); break;
	 case FALSE: printf("FALSE"); break;
	 case IF: printf("IF"); break;
	 case WHILE: printf("WHILE "); break;
	 case ASSIGN: printf("SET"); break;
	 case ADD: printf("+"); break;
	 case SUB: printf("-"); break;
	 case MUL: printf("*"); break;
	 case DIV: printf("/"); break;
	 case MOD: printf("MOD"); break;
	 case SEQ: printf("BEGIN"); break;
	 case POT: printf("EXPT"); break;
	 case NEG: printf("-"); break;
	 case SMALLER: printf("<"); break;
	 case SMALLEREQUAL: printf("<="); break;
	 case GREATER: printf(">"); break;
	 case GREATEREQUAL: printf(">="); break;
	 case NOTEQUAL: printf("<>"); break;
	 case NOT: printf("NOT"); break;
	 case OR: printf("OR"); break;
	 case AND: printf("AND"); break;
	 case EQUAL: printf("="); break;
	 case PUT: printf("PUT"); break;
	 case GET: printf("GET"); break;
	 default: printf("else: %d", tree->type);
        }
  
        
	ParseTreeCell walker = tree->tag.subtrees->first;
	while (walker != NULL)
	{ 
	    printf(" ");
	    inOrder(walker->value);
	    walker = walker->next;
	}
	printf(")");
    }
}



void inOrderCommand(ParseTree tree)
{
    int i;
    char op[10];
    if (tree == NULL)
    {
	printf("NULL");
	return;
    }
    else
    {

        if (tree->type == VARIABLE)
	{
	   printf("%s", SymbolTableFindSymbolById(vartable, tree->tag.value));
	   return;
	}
	if (tree->type == INTEGER)
	{
	   printf("%d", tree->tag.value);
	   return;
	}
	if (tree->type == SKIP)
	{
	  printf("SKIP");
	  return;
	}

	switch (tree->type)
	{
	    case TRUE: printf("TRUE"); break;
	    case FALSE: printf("FALSE"); break;
	    case IF: 
		printf("IF "); 
		inOrderCommand(tree->tag.subtrees->first->value);
		break;
	    case WHILE: 
		printf("WHILE ");
		inOrderCommand(tree->tag.subtrees->first->value);
		break;
	    case ASSIGN: 
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(":=");
		inOrderCommand(tree->tag.subtrees->last->value); 
		break;
	    case ADD: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("+"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;
	    case SUB: 	
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("-"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;
	    case MUL: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("*"); 
		inOrderCommand(tree->tag.subtrees->last->value);
		printf(")");
		break;
	    case DIV:
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("/"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;
	    case MOD: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(" MOD ");
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;
	    case SEQ: printf("\\"); break;
	    case POT: 
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("EXPT"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		break;	    
	    case NEG: 
		printf("-"); 
		inOrderCommand(tree->tag.subtrees->first->value);
		break;
	    case SMALLER: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("<"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;	 
	    case SMALLEREQUAL: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("<="); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;	 
	    case GREATER: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(">"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;	 
	    case GREATEREQUAL:
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(">="); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;	 
	    case NOTEQUAL: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf("<>"); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;	 
	    case NOT:
		printf("NOT "); 
		inOrderCommand(tree->tag.subtrees->first->value);
		break;
	    case OR: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(" OR "); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;
	    case AND: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(" AND "); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;		
	    case EQUAL: 
		printf("(");
		inOrderCommand(tree->tag.subtrees->first->value);
		printf(" = "); 
		inOrderCommand(tree->tag.subtrees->last->value); 
		printf(")");
		break;
	    case PUT: 
		printf("PUT "); 
		inOrderCommand(tree->tag.subtrees->first->value); 
		break;
	    case GET: 
		printf("GET "); 
		inOrderCommand(tree->tag.subtrees->first->value); 
		break;
	    default: printf("?%d?", tree->type);
        }
  
    }
}



