/*****************************
  
  parsetree.c
  
  27.03.2004
  
  Provides procedures for the ADT
  ParseTree
  which is used by the parser
*****************************/


#include "parsetree.h"

/* procedures & functions */

/* auxilliary functions */
int accessible_slots(int type)
{
    if ((type == SKIP) || (type == VARIABLE) || (type == INTEGER))
	return 0;                                                      
    switch (type)
    {
	case IF: return 3;
	case WHILE: return 3;
	case SEQ: return -1;
	case NEG: return 1;
	case NOT: return 1;
	default: return 2;
    }
}


/* ParseTree createParseTree(int type)
   Returns an initialized ParseTree (with
   given type); it is still necessary to
   set the variable id or subtrees.  
   Returns:
   NULL: error (memory)
   else: Pointer to the created ParseTree
*/
ParseTree createParseTree(int type)
{
    ParseTree result;
    if ((result = (ParseTree)malloc(sizeof(struct ParseTree_))) == NULL)
	return NULL;
    else
    {
	result->type = type;
	if (!((type == SKIP) || (type == VARIABLE) || (type == INTEGER) || (type == TRUE) || (type == FALSE)))
	{
	    if ((result->tag.subtrees = (ParseTreeList)malloc(sizeof(struct ParseTreeList_))) == NULL)
	    {
		free(result);
		return NULL;
	    }
	    else
	    {
              result->label = -1;
              result->tag.subtrees->first = NULL;
              result->tag.subtrees->last = NULL;
              result->tag.subtrees->length = 0;
	    }
	}

    }
    return result;
}

/* int setValue(ParseTree tree, int val)
   Sets the id of the variable or the
   value of an integer of the given tree
   Returns:
   -2: tree == NULL
   -1: Error (no tree/node of type variable or integer)
   else: Success   
*/
int setVarId(ParseTree tree, int val)
{
    if (tree == NULL)
	return -2;
    if ((tree->type == VARIABLE) || (tree->type == INTEGER))
    {
	tree->tag.value = val;
	return 1;
    }
    else
	return -1;
}

/* int addSubTree(ParseTree tree, ParseTree subtree, insertmode)
   adds a SubTree (a ParseTree) to the given tree 
   in the next free slot
   insertmode: see addParseTree
   Returns:
   -1: Error (slot not accessible)
   else: Success
*/
int addSubTree(ParseTree tree, ParseTree subtree, int insertmode)
{
    addParseTree(tree->tag.subtrees, subtree, insertmode);
    return 1;
}

/* int ParseTreeList addParseTree(ParseTreeList list, ParseTree tree, int insertmode)
   adds an element in the ParseTreeList, if
   insertmode = FIRST at the beginning of the list,
   = LAST  at the end
   Returns:  -1: error (memory)
             else: success
*/
int addParseTree(ParseTreeList list, ParseTree tree, int insertmode)
{
    ParseTreeCell newelement;
    if ((newelement = (ParseTreeCell)malloc(sizeof(struct ParseTreeCell_))) == NULL)
	    return -1;

    newelement->value = tree;
    
    if (list->first == NULL)  /* no element inserted yet */
    {
	list->first = newelement;
	list->first->next = NULL;
	list->last = list->first;
	list->length = 1;
	return 1;
    }
    if (insertmode == FIRST)
    {
	newelement->next = list->first;
	list->first = newelement;
    }
    else
    {
	list->last->next = newelement;
	newelement->next = NULL;
	list->last = list->last->next;
    }
    list->length++;
    return 1;
}

/* Tree getSubTree(ParseTree tree, int which) 
   returns the which-th subtree of tree
   (if there's no which-th, it returns NULL
*/
ParseTree getSubTree(ParseTree tree, int which)
{
    ParseTreeCell walker = tree->tag.subtrees->first;
    int i;
    for(i = 0; i<which; i++)
    {
	if (walker == NULL) return NULL;
	walker = walker->next;
    }
    return walker->value;
}
