/*****************************
  
  parsetree.h
  
  27.03.2004
  
  Provides structures and
  procedures for the ADT
  ParseTree
  which is used by the parser
*****************************/


#ifndef _parsetree_h

#define _parsetree_h

/* included header files */

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "symbols.h"

/* definitions */
#define FIRST 1
#define LAST 2

/* structures */



typedef union nodetag_
{
    int value;
    struct ParseTreeList_ *subtrees;
} nodetag;

typedef struct ParseTree_
{
    int type;
    int label;
    nodetag tag;
} *ParseTree;

typedef struct ParseTreeCell_
{  
    ParseTree value;
    struct ParseTreeCell_ *next;
} *ParseTreeCell;

typedef struct ParseTreeList_
{
    int length;
    ParseTreeCell first, last;
} *ParseTreeList;


/* y.tab.h */

typedef union {
  ParseTree tree;
  char *label;
} YYSTYPE;
extern YYSTYPE yylval;



/* procedures & functions */

/* ParseTree createParseTree(int type)
   Returns an initialized ParseTree (with
   given type); it is still necessary to
   set the variable id or subtrees.
   Returns:
   NULL: error (memory)
   else: Pointer to the created ParseTree
*/
ParseTree createParseTree(int type);

/* int setVarId(ParseTree tree, int val)
   Sets the id of the variable or the
   value of an integer of the given tree to type
   Returns:
   -1: Error (no tree/node of type variable or integer)
   else: Success   
*/
int setVarId(ParseTree subtree, int val);

/* int addSubTree(ParseTree tree, ParseTree subtree, insertmode)
   adds a SubTree (a ParseTree) to the given tree 
   in the next free slot
   insertmode: see addParseTree
   Returns:
   -1: Error (slot not accessible)
   else: Success
*/
int addSubTree(ParseTree tree, ParseTree subtree, int insertmode);

/* ParseTreeList addParseTree(ParseTreeList list, ParseTree tree, int insertmode)
   adds an element in the ParseTreeList, if
   insertmode = FIRST at the beginning of the list,
              = LAST  at the end
   Returns:  -1: error (memory)
             else: success
*/
int addParseTree(ParseTreeList list, ParseTree tree, int insertmode);

/* Tree getSubTree(ParseTree tree, int which) 
   returns the which-th subtree of tree
   (if there's no which-th, it returns NULL
*/
ParseTree getSubTree(ParseTree tree, int which);
 
#endif
