/*****************************
  
  symboltable.c
  
  26.03.2004
  
  Provides an ADT SymbolTable
  which is used by the lexer
*****************************/

/* included header files */

#include "symboltable.h"


/* procedures & functions */

/* SymbolTable createSymbolTable()
   Initializes a SymbolTable 
*/
SymbolTable createSymbolTable()
{
    SymbolTable result;
    if ((result = (SymbolTable)malloc(sizeof(struct SymbolTable_))) == NULL)
	return NULL;
    else 
    {
	result->length = 0;
	result->entries = NULL;
	return result;
    }
}

/* int SymbolTableAddSymbol(SymbolTable t, char *symbol)
   adds a Symbol to the SymbolTable (if there's
   already a symbol with the same name, nothing will
   happen)
   Returns:
    -1    (error)
    else  id of the Symbol
*/
int SymbolTableAddSymbol(SymbolTable t, char *symbol)
{
    if (t->entries == NULL)
    {
	if ((t->entries = (SymbolCell)malloc(sizeof(struct SymbolCell_))) == NULL)
	    return -1;
	else
	{
	    t->entries->id = 1;
	    t->entries->name = symbol;
	    t->entries->next = NULL;
	    t->length = 1;
	    return 1;
	}
    }
    else
    {
	int cmp;
	SymbolCell walker, buffer;
	/* it can be necessary to insert the symbol at the first
	   position */
	if (strcmp(symbol, t->entries->name) < 0)
	{
	    if ((walker = (SymbolCell)malloc(sizeof(struct SymbolCell_))) == NULL)
		return -1;
	    else
	    {  
		walker->name = symbol;
		t->length++;
		walker->id = t->length;
		walker->next = t->entries;
		t->entries = walker;
		return t->length;
	    }
	}
	else if (strcmp(symbol, t->entries->name) == 0)
	    return t->entries->id;
	else
	{
	    walker = t->entries;
	    while (walker->next != NULL)
	    {
		cmp = strcmp(symbol, walker->next->name);
		if (cmp < 0) 
		{  
		    if ((buffer = (SymbolCell)malloc(sizeof(struct SymbolCell_))) == NULL)
			return -1;
		    else
		    {
			buffer->next = walker->next;
			buffer->name = symbol;
			t->length++;
			buffer->id = t->length;
			walker->next = buffer;
			return buffer->id;
		    }
		}
		else if (cmp == 0)
		{
		    return walker->next->id;
		}
		else
		    walker = walker->next;
	    }
	    /* At this point, we have to insert the symbol at the end of the table */
	    if ((walker->next = (SymbolCell)malloc(sizeof(struct SymbolCell_))) == NULL)
		return -1;
	    else
	    {
		walker->next->next = NULL;
		walker->next->name = symbol;
		t->length++;
		walker->next->id = t->length;
		return t->length;
	    }	
	}
    }
}

/* int SymbolTableFindSymbolByName(SymbolTable t, char *symbol)
   Returns:
    -1 = symbol not found
     x = id of symbol in t
*/
int SymbolTableFindSymbolByName(SymbolTable t, char *symbol)
{
    int cmp;
    SymbolCell walker = t->entries;
    while (walker != NULL)
    {
	cmp = strcmp(symbol, walker->name);
	if (cmp == 0)
	    return walker->id;
	/*else if (cmp < 0) */
	    walker = walker->next;
            /* else return -1; */
    }
    return -1;
}

/* char *SymbolTableFindSymbolById(SymbolTable t, int id)
   Returns:
    NULL = symbol not found
    else = name of symbol 
*/
char *SymbolTableFindSymbolById(SymbolTable t, int id)
{
    SymbolCell walker = t->entries;
    if (id > t->length) return NULL;
    while (walker != NULL)
    {
	if (walker->id == id)
	    return walker->name;
	else
	    walker = walker->next;
    }
    return NULL;
}

/* int SymbolTablesetValue(SymbolTable t, int id)
   Returns:
    -1 = symbol not found
    else = ok 
*/
int SymbolTableSetValue(SymbolTable t, int id, int val)
{
    SymbolCell walker = t->entries;
    while (walker != NULL)
    {
	if (walker->id == id)
	{
	    walker->value = val;
	    return 1;
	} 
	walker = walker->next;
    }
    return 1;
}


/* int SymbolTableGetValue(SymbolTable t, int id)
   Returns:
    Value of the symbol (0 if not found)
*/
int SymbolTableGetValue(SymbolTable t, int id)
{
    SymbolCell walker = t->entries;
    while (walker != NULL)
    {
	if (walker->id == id)
	{
	    return(walker->value);
	} 
	walker = walker->next;
    }
    return 0;
}

/* void toLower(char * text)
    all upper case letters to lower case
*/
void toLower(char *text)
{
    while (*text != '\0')
    {
	*text = (char)tolower((int)*text);
	text++;
    }
}
