/*****************************
  
  symboltable.h
  
  26.03.2004
  
  Provides an ADT SymbolTable
  which is used by the lexer
*****************************/

/* included header files */

#ifndef _symboltable_h

#define _symboltable_h

#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#define MAX_LEN_VAR 35

/* structures */

/*
 SymbolCell
 structure for a single entriex
 of the SymbolTable
*/
typedef struct SymbolCell_
{
    char *name;
    int id;
    int value;
    struct SymbolCell_ *next;
} *SymbolCell;

/*
 SymbolTable
 structure for a simple linked
 string list 
*/
typedef struct SymbolTable_
{
    int length;
    SymbolCell entries;
} *SymbolTable;



/* procedures & functions */

/* SymbolTable createSymbolTable()
   Initializes a SymbolTable 
*/
SymbolTable createSymbolTable();

/* int addSymbol(SymbolTable t, char *symbol)
   adds a Symbol to the SymbolTable (if there's
   already a symbol with the same name, nothing will
   happen)
   Returns:
    -1 = error
     1 = insertion successful
     2 = no insertion (already in)
*/
int SymbolTableAddSymbol(SymbolTable t, char *symbol);

/* int SymbolTablefindSymbolByName(SymbolTable t, char *symbol)
   Returns:
    -1 = symbol not found
     x = id of symbol in t
*/
int SymbolTableFindSymbolByName(SymbolTable t, char *symbol);

/* char *SymbolTableFindSymbolById(SymbolTable t, int id)
   Returns:
    NULL = symbol not found
    else = name of symbol 
*/
char *SymbolTableFindSymbolById(SymbolTable t, int id);

/* int SymbolTableSetValue(SymbolTable t, int id)
   Returns:
    -1 = symbol not found
    else = ok 
*/
int SymbolTableSetValue(SymbolTable t, int id, int val);

/* int SymbolTableGetValue(SymbolTable t, int id)
   Returns:
    Value of the symbol (0 if not found)
*/
int SymbolTableGetValue(SymbolTable t, int id);

/* void toLower(char * text)
    all upper case letters to lower case
*/
void toLower(char *text);

#endif
