/*  textctrl.c

    Text-I/O procedures
    
    02.08.2004
*/

#include "textctrl.h"


extern SymbolTable vartable;
extern LabelList labellist;
extern int parsing_ok;
extern GenericList fwlist;
extern FlowGraph graph;
extern ParseTree parsetree;
extern FrameWork actual_fw;
extern FlowRelationList worklist;
extern int stepmode;

 
char empty[] = {""};

char * nn(char *s)
{
    if (s == NULL) return empty;
    else return s;
}

void print_welcome_msg()
{
  printf("\n**************************************************\n");
  printf("*               wAnalyzer v 0.1                  *\n");
  printf("**************************************************\n");
  printf("*                                                *\n");
  printf("*                                                *\n");
  printf("*    Softwarepraktikum an der Uni Stuttgart      *\n");
  printf("**************************************************\n\n");
}


void doProgramAnalysis(int choice)
{

  if (stepmode)
    {
      /* calling the algorithm "step by step" */
      setActualFrameWork(choice);  
      fwtools_init();  
      initWorkList(graph, getActualFrameWork());
      printFlowGraphAnalysis(graph, getActualFrameWork());
      while (stepWorkList(graph, getActualFrameWork()) != 1)
	{
	  printFlowGraphAnalysis(graph, getActualFrameWork());
	  printf("Beliebige Taste für weiter");
	  getc(stdin);
	}
      printf("Worklistalgorithmus beendet. \n");
      printFlowGraphAnalysis(graph, getActualFrameWork()); 
    }
  else
    {
      setActualFrameWork(choice);  
      fwtools_init();
      initWorkList(graph, getActualFrameWork());
      while (stepWorkList(graph, getActualFrameWork()) != 1)
	{} 
      printFlowGraphAnalysis(graph, getActualFrameWork());
    }
}


int printFrameWorks()
{
  int i = 0;
  char *buffer;
  GenericCell walker = fwlist->first;
  while (walker != NULL)
    {
      printf("[%d]:%s\n", i, ((FrameWorkEntry)(walker->value))->name);
      walker = walker->next;
      i++;
    }
  return i-1;
}


void printFlowGraphAnalysis(FlowGraph g, FrameWork fw)
{
    Element a;
    FlowNodeList w;
    char *name, *buffer;
    w = g->all;
    while (w != NULL)
    {
      name = findLabelbyID(labellist, w->node->label);
      if (name != NULL) 
        {
          printf("Knoten %s [%d]\n", name, w->node->label);
          if (fw->direction != 1)
            {
              printf("Analyse:\n");
              buffer = (fw->printfunc)(w->node->analysis);
              /* direction */
              printf("IN: %s", buffer);
              free(buffer);
              printf("\n");
              a = fw->ref((void *)(fw->transfer)(w->node, (VarSet)w->node->analysis));
              buffer = (fw->printfunc)(a);
              printf("OUT: %s", buffer);
              free(buffer);
              printf("\n");         
              printf("\n");
              fw->deref(a);
            }
          else
            {
             printf("Analyse:\n");
              a = fw->ref((void *)(fw->transfer)(w->node, (VarSet)w->node->analysis));
              buffer = (fw->printfunc)(a);
              printf("IN: %s", buffer);
              free(buffer);
              printf("\n");   
              buffer = (fw->printfunc)(w->node->analysis);
              /* direction */
              printf("OUT: %s", buffer);
              free(buffer);
              printf("\n");
              printf("\n");
              fw->deref(a);             
            }
        }
      w = w->next;
    }
}


void textcontrol(char *filename)
{
  int choice, maxchoice;

  print_welcome_msg();
  
  if (parseFile(filename) == 0)
    {
      printf("Fehler beim Parsen. Programm wird beendet.\n");
      return;
    }
  
  printf("Flussgraph wird erstellt...\n");
  graph = generateFlowGraph(parsetree);
  if (graph == NULL) 
    {
      printf("Fehler beim Erstellen des Flussgraphen. Programm wird beendet.\n");
      return;
    }
  
  /* initialization of the framework list */
  init_fwlist();  
  
  printf("Bitte waehlen Sie eine Analyseart: \n");
  maxchoice = printFrameWorks();
  printf(">");
  scanf("%d", &choice);
  if (choice <= maxchoice)
    doProgramAnalysis(choice);
  else
    {
      printf("Ungültige Eingabe.");
    }

  
  /* optional: dot-Output */
  

}
