/*
  varset.c
  
  ADT for handling sets of Variables
  
  18.06.2004  
*/


#include "varset.h"

/*
  VarSet createVarSet()
  returns the null set
*/
VarSet createVarSet()
{
  return NULL;
}

/*
  int VarSetElementCount(VarSet set)
  Returns |set|
*/
int VarSetElementCount(VarSet set)
{
  int result = 0;
  while (set != NULL)
    {
      result++;
      set = set->next;
    }
  return result;
}

/*
  VarSet VarSetaddElementbyName(VarSet set, char *name)
  Adds the variable "name" to set; if name doesn't exist in the
  symboltable, nothing will happen.
*/
VarSet VarSetaddElementbyName(VarSet set, char *name)
{
  int id = SymbolTableFindSymbolByName(vartable, name);
  if (id == -1) 
    return set;
  else 
    return VarSetaddElement(set, id);
}

/*
  VarSet VarSetremoveElementbyName(VarSet set, char *name)
  Removes the variable "name" from the set; if name doesn't exist in
  the set or in the symboltable, nothing will happen.
*/
VarSet VarSetremoveElementbyName(VarSet set, char *name)
{
  
  int id = SymbolTableFindSymbolByName(vartable, name);
  if (id == -1) 
    return set;
  else 
    return VarSetremoveElement(set, id);
}

/*
  VarSet VarSetaddElement(VarSet set, int id)
  Adds the variable referenced by id to the set
*/
VarSet VarSetaddElement(VarSet set, int id)
{
  VarSet newSet, walker1, walker2;
  
	if (set == NULL)
	{
	    /* create new entry */
	    newSet = (VarSet)(malloc(sizeof(struct VarSet_)));
	    newSet->val = id;
	    newSet->next = NULL;
	    return newSet;
	}
/* special case: insertion at the beginning of the list */
  if (set->val > id)
    {      
      /* create new entry */
      newSet = (VarSet)(malloc(sizeof(struct VarSet_)));
      if (newSet == NULL)
        return NULL;
      newSet->val = id;
      newSet->next = set;
      return newSet;
    }
  else if (set->val == id)
    return set;
    
  walker1 = set;
  walker2 = set;
  
  /* the element has to be within the list or at the end of the list */
  while (walker1 != NULL)
    {
      if (walker1->val > id)
        {
          /* create new entry */
          newSet = (VarSet)(malloc(sizeof(struct VarSet_)));
          if (newSet == NULL)
            return NULL;
          newSet->val = id;
          walker2->next = newSet;
          newSet->next = walker1;
          return set;
        }
      else if (walker1->val == id)
      {
        return set;
      }
      else
        {
          walker2 = walker1;
          walker1 = walker1->next;
        }
    }
  /* else, the label has to be inserted at the end of the list */
  /* create new entry */
  newSet = (VarSet)(malloc(sizeof(struct VarSet_)));
  if (newSet == NULL)
    return NULL;
  newSet->val = id;
  newSet->next = NULL;
  walker2->next = newSet;
  return set;
}

/*
  VarSet VarSetremoveElement(VarSet set, int id)
  Removes the Element referenced by id from the set;
  if it isn't in the set, nothing will happen
*/
VarSet VarSetremoveElement(VarSet set, int id)
{
    VarSet walker = set;
    VarSet buffer;
    if (set == NULL)
	return NULL;   
	
    if (set->next == NULL)
    {
	if (set->val == id)
	{
	    free(set);
	    return NULL;
	}
	else
	    return set;
    }
    if (set->val == id)
    {

	buffer = set->next;
	free(set);
	return buffer;
    }
    while (walker->next != NULL)
    {
	if (walker->next->val == id)
	{	
	    buffer = walker->next;
	    walker->next = walker->next->next;
	    free(buffer);
	    return set;
	}
	else if (walker->next->val > id)
	    return set;
	else
	    walker = walker->next;
    }
    return set;  
}

/*
  VarSet VarSetMerging(VarSet a, VarSet b)
  Merges the two sets a and b, and returns the resulting set.
  !!! the new list has no common elements with the sets a and b !!!
 */
VarSet VarSetMerge(VarSet a, VarSet b)
{
  VarSet walker = a;
  VarSet result = createVarSet();
    
  while (walker != NULL)
    {
      result = VarSetaddElement(result, walker->val);
      walker = walker->next;
    }
  walker = b;
  while (walker != NULL)
    {
      result = VarSetaddElement(result, walker->val);
      walker = walker->next;  
      
    }
  return result;  
}




/*
  VarSet VarSetIntersection(VarSet a, VarSet b)
  Builds the intersection of a and b
  !!! the new list has no common elements with the sets a and b !!!
 */
VarSet VarSetIntersection (VarSet a, VarSet b)
{
    VarSet walker = a;
    VarSet result = createVarSet();
    while (walker != NULL)
    {
	if (VarSetIsIn(b, walker->val))
	    result = VarSetaddElement(result, walker->val);
	walker = walker->next;
    }
    return result;
}


int VarSetIsIn(VarSet set, int id)
{
    VarSet walker = set;
    while (walker != NULL)
    {
	if (walker->val == id)
	    return 1;
	else if (walker->val > id)
	    return 0;
	else
	    walker = walker->next;
    }
    return 0;
}

void VarSetDestroy(VarSet set)
{
  VarSet walker = set;
  while (set != NULL)
    {
      walker = set->next;
      free(set);
      set = walker;
    }
}

void VarSetPrint(VarSet set)
{
  printf("< ");
    while (set != NULL)
    {
	printf ("%s ", SymbolTableFindSymbolById(vartable, set->val));
	set = set->next;
    }
    printf(" >\n");
}

char *VarSetSPrint(VarSet set)
{ 
  char *result = (char *)malloc((sizeof(char))*(VarSetElementCount(set)*(2 + MAX_LEN_VAR) + 5));
  char *buffer;
  int i = 0;
  result[i] = '(';
  i++;
  while( set != NULL)
    {
      buffer = SymbolTableFindSymbolById(vartable, set->val);
      sprintf(result+i, "%s ", buffer);
      i += strlen(buffer);
      buffer[i] = ' ';
      i++;
      set = set->next;
    }
  result[i-(i!=1)] = ')';
  result[i+(i==1)] = '\0';
  return result;
}
 

int VarSetEqual(VarSet set1, VarSet set2)
{
    if (set1 == NULL)
	return (set2 == NULL);
    if (set2 == NULL)
	return (set1 == NULL);

    while ((set1 != NULL)&&(set2 != NULL))
    {
	if (set1->val != set2->val)
	    return 0;
	else
	{
	    set1 = set1->next;
	    set2 = set2->next;
	}
    }
    return (set1 == NULL) && (set2 == NULL);
}
