/*
  varset.h
  
  ADT for handling sets of Variables
  
  18.06.2004  
*/


#ifndef _VARSET_H
#define _VARSET_H

#include <stdlib.h>
#include <stdio.h>
#include "symboltable.h"

extern SymbolTable vartable;


typedef struct VarSet_
{
  int val;
  struct VarSet_ *next;
}*VarSet;


/*
  VarSet createVarSet()
  returns the null set
*/
VarSet createVarSet();

/*
  VarSet VarSetElementCount(VarSet set)
  Returns |set|
*/
int VarSetElementCount(VarSet set);

/*
  VarSet VarSetaddElementbyName(VarSet set, char *name)
  Adds the variable "name" to set; if name doesn't exist in the
  symboltable, nothing will happen.
*/
VarSet VarSetaddElementbyName(VarSet set, char *name);

/*
  VarSet VarSetremoveElementbyName(VarSet set, char *name)
  Removes the variable "name" from the set; if name doesn't exist in
  the set or in the symboltable, nothing will happen.
*/
VarSet VarSetremoveElementbyName(VarSet set, char *name);

/*
  VarSet VarSetaddElement(VarSet set, int id)
  Adds the variable referenced by id to the set
*/
VarSet VarSetaddElement(VarSet set, int id);

/*
  VarSet VarSetremoveElement(VarSet set, int id)
  Removes the Element referenced by id from the set;
  if it isn't in the set, nothing will happen
*/
VarSet VarSetremoveElement(VarSet set, int id);


/*
  VarSet VarSetMerge(VarSet a, VarSet b)
  Merges the two sets a and b, and returns the resulting set.
  !!! the new list has no common elements with the sets a and b !!!
 */
VarSet VarSetMerge(VarSet a, VarSet b);


/*
  VarSet VarSetIntersection(VarSet a, VarSet b)
  Builds the intersection of a and b
  !!! the new list has no common elements with the sets a and b !!!
 */
VarSet VarSetIntersection (VarSet a, VarSet b);

/*
  int VarSetIsIn(VarSet set, int id)
  Returns 1 if the variable referenced by id is in the VarSet
  else 0
*/
int VarSetIsIn(VarSet set, int id);

/*
  void VarSetDestroy(VarSet set)
  Destroys the given Set, and frees the used memory
*/
void VarSetDestroy(VarSet set);

/*
  void VarSetPrint(VarSet set)
  Prints the given set to stdin,
  Format: { var1 var2 var3 ... }
*/
void VarSetPrint(VarSet set);

/* int VarSetEqual(VarSet set1, VarSet set2)
   returns 1 if set1 == set2 (set1 <= set2 && set1 >= set2)
*/
int VarSetEqual(VarSet set1, VarSet set2);

/* char *VarSetSPrint(VarSet set)
   Same as VarSetPrint, but "pipes" the output to a string
*/
char *VarSetSPrint(VarSet set);

#endif
