/* worklist.c
   11.06.2004
   Provides the worklist algorithm
   for the data flow analysis
*/

#include "worklist.h"


FlowRelationList worklist;

void printList(FlowRelationList l)
{
    Element a,b;  
  if (l == NULL)
    return;
  else
    {
      printf("(%d, %d)\n", 
             l->value.from->label,
             l->value.to->label
	  ) ;
      
      printList(l->next);
    }
}


/*
  void initWorkList(FlowGraph f, FrameWork fw)
  initalizes the WorkList
*/
int initWorkList(FlowGraph f, FrameWork fw)
{
  FlowNodeList walker;
  FlowRelationElement buffer;
  
  /* first, initialization routine of the framework */
  if (!fw->initfunc()) return 0; 
  
  /* initializing the values of the FlowNodes */

  /* preparing the Work list */
  
  worklist = FlowRelationListCreate(f, fw->direction);

  walker = f->all;
  
  while (walker != NULL)
    {
      walker->node->analysis = fw->ref((Element)((fw->bottom)()));
      walker = walker->next;
    }

  if (fw->direction != 1)
    f->entry->analysis = fw->ref(fw->iota());
  else
    {
      walker = f->exit;
      while (walker != NULL)
	{ 
          walker->node->analysis = fw->ref(fw->iota());
          walker = walker->next;
	}
    }
  return 1;
}


/*
  int stepWorkList(FlowGraph f, FrameWork fw)
  makes one step of the worklist algorithm:
  returns 1: if the algorithm has finished
          0: else
*/
int stepWorkList(FlowGraph f, FrameWork fw)
{
  FlowNodeList walker;
  Element t_ana, supremum, ebackup;
  FlowRelationList backup;
  FlowRelationElement buffer;

  /* no elements in the worklist -> end of algorithm */
  if (FlowRelationListNull(worklist)) return 1;
      
  buffer = FlowRelationListFirst(worklist);
  
  /* freeing the first element */
  backup = FlowRelationListRest(worklist);
  free(worklist);
  worklist = backup; 
  
  /* creating the entry / exit analysis */
  t_ana = fw->ref((Element)(fw->transfer)(buffer.from, buffer.from->analysis));
  /* is the analysis value ok ? */ 
  supremum =  fw->ref((Element)((fw->supremum)(t_ana, buffer.to->analysis)));
  
  if (!(int)(fw->eq)(
	  supremum,
	  buffer.to->analysis)
      )
  {
      /* Correcting the analysis */	 
      /* @@@ problems with deref'ing ? */
      ebackup = fw->ref((Element)(fw->supremum) (
                                                t_ana,
                                                buffer.to->analysis));
      fw->deref(buffer.to->analysis);
      buffer.to->analysis = fw->ref(ebackup);
      

      if (fw->direction == 1)
      { 
        walker = buffer.to->entry;
        buffer.from = buffer.to;
        while (walker != NULL)
	  {
	      buffer.to = walker->node;
	      worklist = FlowRelationListCons(buffer, worklist);
	      walker = walker->next;
	  }  
      }
      else
      {
	  walker = buffer.to->exit;
	  buffer.from = buffer.to;
	  while (walker != NULL)
	  {
            buffer.to = walker->node;
            worklist = FlowRelationListCons(buffer, worklist);
            walker = walker->next;
	  }
      }
  }
  else
      (void)(fw->deref)(t_ana);
  (void)(fw->deref)(supremum);
  return 0;
}


